package services

import (
	"errors"
	"gitea.zjmud.xyz/phyer/rbac/models"
	"gitea.zjmud.xyz/phyer/rbac/repositories"
)

// AssignRoleToUser 为用户分配角色
func AssignRoleToUser(userID string, roleID uint) error {
	// 检查用户是否存在
	user, err := repositories.GetUserByID(userID)
	if err != nil {
		return errors.New("user not found")
	}

	// 检查角色是否存在
	role, err := repositories.GetRoleByID(roleID)
	if err != nil {
		return errors.New("role not found")
	}

	// 分配角色
	return repositories.AssignRoleToUser(user.ID, role.ID)
}

// RemoveRoleFromUser 移除用户的角色
func RemoveRoleFromUser(userID string, roleID uint) error {
	// 检查用户是否存在
	user, err := repositories.GetUserByID(userID)
	if err != nil {
		return errors.New("user not found")
	}

	// 检查角色是否存在
	role, err := repositories.GetRoleByID(roleID)
	if err != nil {
		return errors.New("role not found")
	}

	// 移除角色
	return repositories.RemoveRoleFromUser(user.ID, role.ID)
}

// CreateRoleWithPermissions 创建角色并分配权限
func CreateRoleWithPermissions(roleName string, permissionIDs []uint) (*models.Role, error) {
	// 检查角色是否已存在
	exists, err := repositories.CheckRoleExists(roleName)
	if err != nil {
		return nil, err
	}
	if exists {
		return nil, errors.New("role already exists")
	}

	// 创建角色
	role := &models.Role{Name: roleName}
	role, err = repositories.CreateRole(role)
	if err != nil {
		return nil, err
	}

	// 为角色分配权限
	for _, permissionID := range permissionIDs {
		err = repositories.AddPermissionToRole(role.ID, permissionID)
		if err != nil {
			return nil, err
		}
	}

	return role, nil
}

// CheckUserPermission 检查用户是否具有特定权限
func CheckUserPermission(userID string, permission string) (bool, error) {
	// 获取用户的所有角色
	user, err := repositories.GetUserByID(userID)
	if err != nil {
		return false, err
	}

	// 检查每个角色的权限
	for _, role := range user.Roles {
		for _, perm := range role.Permissions {
			if perm.Name == permission {
				return true, nil
			}
		}
	}

	return false, nil
}

// GetUserPermissions 获取用户的所有权限
func GetUserPermissions(userID string) ([]string, error) {
	var permissions []string
	user, err := repositories.GetUserByID(userID)
	if err != nil {
		return nil, err
	}

	// 收集所有角色的权限
	for _, role := range user.Roles {
		for _, perm := range role.Permissions {
			permissions = append(permissions, perm.Name)
		}
	}

	return permissions, nil
}
